﻿using Correo.Mailgun;
using Correo.MailKit;
using Correo.NetSmtpClient;
using Correo.SendGrid;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using System;

namespace Correo.Extensions
{
    public static class SmtpExtensions
    {
        public static void AddSmtpService(this IServiceCollection services, IConfiguration configuration)
        {
            var mediator = configuration.GetValue("SmtpMediator", "SmtpClient");
            if (mediator.EqualsIgnoreCase("SmtpClient"))
            {
                services.AddSmtpClient(configuration.GetSection("SmtpClient"));
            }
            else if (mediator.EqualsIgnoreCase("SendGrid"))
            {
                services.AddSendGridService(configuration.GetSection("SendGrid"));
            }
            else if (mediator.EqualsIgnoreCase("Mailgun"))
            {
                services.AddMailgunService(configuration.GetSection("Mailgun"));
            }
            else
            {
                throw new NotImplementedException($"SmtpMediator '{mediator}' is not implemented.");
            }
        }

        private static void AddSmtpClient(this IServiceCollection services, IConfiguration configuration)
        {
            var package = configuration.GetValue("Package", ".NET");
            if (package.EqualsIgnoreCase(".NET"))
            {
                services.AddNetSmtpClient(configuration);
            }
            else if (package.EqualsIgnoreCase("MailKit"))
            {
                services.AddMailKitSmtpClient(configuration);
            }
            else
            {
                throw new NotImplementedException($"SmtpClient:Package '{package}' is not implemented.");
            }
        }
    }
}
